#include <QCoreApplication>
#include <QDir>
#include <glacierapp.h>

inline bool isRunningInstalled()
{
    return (QCoreApplication::applicationDirPath() == "/usr/bin");
}

inline QString appDirectory()
{
    bool installed = isRunningInstalled();
    QString path = QString(QCoreApplication::applicationDirPath() + "/qml/");
    if (installed) {
        path = QString("/usr/share/%1/qml/").arg(qApp->applicationName());
    }
    qCDebug(lcGlacierAppCoreLog) << "isRunningInstalled" << installed << path;
    return path;
}
