/*
 * Copyright (C) 2023-2024 Chupligin Sergey <neochapay@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "dbusadaptor.h"
#include "glacierapp.h"
#include "qcoreapplication.h"

#include <QDBusConnection>

DBusAdaptor::DBusAdaptor(QQuickWindow* window)
    : QDBusAbstractAdaptor(window)
    , m_window(window)
{
    QString serviceName = QString("org.glacier.%1").arg(qApp->applicationName().replace("-", "_"));

    QDBusConnection::sessionBus().registerService(serviceName);
    if (!QDBusConnection::sessionBus().registerObject("/", m_window)) {
        qCWarning(lcGlacierAppCoreLog) << Q_FUNC_INFO << "Cannot register DBus object!";
    };
}

void DBusAdaptor::show(QStringList args)
{
    Q_UNUSED(args);
    qCDebug(lcGlacierAppCoreLog) << "show window from dbus call";
    m_window->show();
}

void DBusAdaptor::quit(QStringList args)
{
    Q_UNUSED(args);
    qCDebug(lcGlacierAppCoreLog) << "quit over dbus";
    qApp->quit();
}
